package com.cnbm.basic.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.admin.params.IdParam;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.basic.dto.PlatformDTO;
import com.cnbm.basic.excel.PlatformExcel;
import com.cnbm.basic.service.IPlatformService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 站点表  前端控制器
 *
 * @author why
 * @since  2022-07-13
 */
@RestController
@RequestMapping("/basic/platform")
@Api(tags="站点表")
public class PlatformController {
    @Autowired
    private IPlatformService platformService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "name", value = "名称", paramType = "query", dataTypeClass = String.class),
        @ApiImplicitParam(name = "code", value = "编码", paramType = "query", dataTypeClass = String.class),
        @ApiImplicitParam(name = "status", value = "状态", paramType = "query", dataTypeClass = Integer.class)
    })
    @PreAuthorize("@ex.hasAuthority('basic:platform:page')")
    public Result<PageData<PlatformDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<PlatformDTO> page = platformService.page(params);

        return new Result<PageData<PlatformDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('basic:platform:info')")
    public Result<PlatformDTO> get(@PathVariable("id") Long id){
        PlatformDTO data = platformService.get(id);

        return new Result<PlatformDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('basic:platform:save')")
    public Result save(@RequestBody PlatformDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        platformService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('basic:platform:update')")
    public Result update(@RequestBody PlatformDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        platformService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('basic:platform:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        platformService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('basic:platform:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<PlatformDTO> list = platformService.list(params);

        ExcelUtils.exportExcelToTarget(response, "Platform", list, PlatformExcel.class);
    }

    @PostMapping("status")
    @ApiOperation("改变状态")
    @LogOperation("改变状态")
    public Result changeStatus(@RequestBody IdParam id){
        platformService.changeStatus(id.getId());

        return new Result();
    }

}