package com.cnbm.basic.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.basic.dto.ProductFeaturesHisDTO;
import com.cnbm.basic.excel.ProductFeaturesHisExcel;
import com.cnbm.basic.service.IProductFeaturesHisService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 产品特性 历史表  前端控制器
 *
 * @author why
 * @since  2022-07-15
 */
@RestController
@RequestMapping("/basic/productFeaturesHis")
@Api(tags="产品特性 历史表")
public class ProductFeaturesHisController {
    @Autowired
    private IProductFeaturesHisService productFeaturesHisService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "id", value = "id", paramType = "query", dataTypeClass = Long.class)
    })
    @PreAuthorize("@ex.hasAuthority('basic:productFeaturesHis:page')")
    public Result<PageData<ProductFeaturesHisDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<ProductFeaturesHisDTO> page = productFeaturesHisService.page(params);

        return new Result<PageData<ProductFeaturesHisDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('basic:productFeaturesHis:info')")
    public Result<ProductFeaturesHisDTO> get(@PathVariable("id") Long id){
        ProductFeaturesHisDTO data = productFeaturesHisService.get(id);

        return new Result<ProductFeaturesHisDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('basic:productFeaturesHis:save')")
    public Result save(@RequestBody ProductFeaturesHisDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        productFeaturesHisService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('basic:productFeaturesHis:update')")
    public Result update(@RequestBody ProductFeaturesHisDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        productFeaturesHisService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('basic:productFeaturesHis:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        productFeaturesHisService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('basic:productFeaturesHis:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<ProductFeaturesHisDTO> list = productFeaturesHisService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, ProductFeaturesHisExcel.class);
    }

}