package com.cnbm.basic.controller;

import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.admin.params.IdParam;
import com.cnbm.basic.dto.SequenceDTO;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.basic.dto.ProductWorkingprocedureRelationDTO;
import com.cnbm.basic.excel.ProductWorkingprocedureRelationExcel;
import com.cnbm.basic.service.IProductWorkingprocedureRelationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 工序 表  前端控制器
 *
 * @author why
 * @since  2022-07-15
 */
@RestController
@RequestMapping("/basic/productWorkingprocedureRelation")
@Api(tags="产品-工序 关系表")
public class ProductWorkingprocedureRelationController {
    @Autowired
    private IProductWorkingprocedureRelationService productWorkingprocedureRelationService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "id", value = "id", paramType = "query", dataTypeClass = Long.class)
    })
    @PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:page')")
    public Result<PageData<ProductWorkingprocedureRelationDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<ProductWorkingprocedureRelationDTO> page = productWorkingprocedureRelationService.page(params);

        return new Result<PageData<ProductWorkingprocedureRelationDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:info')")
    public Result<ProductWorkingprocedureRelationDTO> get(@PathVariable("id") Long id){
        ProductWorkingprocedureRelationDTO data = productWorkingprocedureRelationService.get(id);

        return new Result<ProductWorkingprocedureRelationDTO>().ok(data);
    }

    @PostMapping("save")
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:save')")
    public Result save(@RequestBody ProductWorkingprocedureRelationDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        productWorkingprocedureRelationService.save(dto);

        if(ObjectUtils.isNotNull(dto.getWorkingProcedureId())) {
            return new Result();
        }
        else{
            return new Result().error(1,"该工艺已经绑定");
        }
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:update')")
    public Result update(@RequestBody ProductWorkingprocedureRelationDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        productWorkingprocedureRelationService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        productWorkingprocedureRelationService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<ProductWorkingprocedureRelationDTO> list = productWorkingprocedureRelationService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, ProductWorkingprocedureRelationExcel.class);
    }

    @GetMapping("getWorkingprocedureByProductId/{productId}")
    @ApiOperation("查询所有工序 依据产品id")
    @LogOperation("查询所有工序 依据产品id")
    public Result<List<ProductWorkingprocedureRelationDTO>> getWorkingprocedureByProductId(@PathVariable("productId") Long productId){
        List<ProductWorkingprocedureRelationDTO> list = productWorkingprocedureRelationService.getWorkingprocedureByProductId(productId);
        return new Result<List<ProductWorkingprocedureRelationDTO>>().ok(list);
    }

    @PostMapping("batchInsertWorkingprocedure")
    @ApiOperation("添加工序流程")
    @LogOperation("添加工序流程")
    //@PreAuthorize("@ex.hasAuthority('basic:productWorkingprocedureRelation:batchInsertWorkingprocedure')")
    public Result batchInsertWorkingprocedure(@RequestBody ProductWorkingprocedureRelationDTO[] lists){

        productWorkingprocedureRelationService.batchInsertWorkingprocedure(lists);

        return new Result();
    }

    @PutMapping("sequenceUp")
    @ApiOperation("产品工序上移")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "productId", value = "产品", paramType = "query", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "index", value = "产品工序序号", paramType = "query", dataTypeClass = Integer.class)
//    })
    public Result sequenceUp(@RequestBody SequenceDTO sequenceDTO){

        productWorkingprocedureRelationService.sequenceUp(sequenceDTO);

        return new Result();
    }

    @PutMapping("sequenceDown")
    @ApiOperation("产品工序下移")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "productId", value = "产品", paramType = "query", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "index", value = "产品工序序号", paramType = "query", dataTypeClass = Integer.class)
//    })
    public Result sequenceDown(@RequestBody SequenceDTO sequenceDTO){

        productWorkingprocedureRelationService.sequenceDown(sequenceDTO);

        return new Result();
    }

}