package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.basic.dto.MachineDTO;
import com.cnbm.basic.dto.ProductTypeDTO;
import com.cnbm.basic.entity.ProductType;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.FactoryDTO;
import com.cnbm.basic.mapper.FactoryMapper;
import com.cnbm.basic.entity.Factory;
import com.cnbm.basic.service.IFactoryService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 工厂 表
 *
 * @author why
 * @since 2022-07-13
 */
@Service
public class FactoryServiceImpl extends CrudServiceImpl<FactoryMapper, Factory, FactoryDTO> implements IFactoryService {
@Autowired private FactoryMapper mapper;

    @Override
    public QueryWrapper<Factory> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");
        String type = (String)params.get("type");

        QueryWrapper<Factory> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);
        wrapper.like(StringUtils.isNotBlank(type), "type", type);
        wrapper.eq(ObjectUtils.isNotNull(params.get("status")), "status", params.get("status"));

        return wrapper;
    }

    @Override
    public PageData<FactoryDTO> page (Map<String, Object> params){
        IPage<Factory> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, FactoryDTO.class);
    }

    @Override
    public FactoryDTO get(Long id) {
        Factory entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, FactoryDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(FactoryDTO dto) {
        Factory entity = ConvertUtils.sourceToTarget(dto, Factory.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(FactoryDTO dto) {
        Factory entity = ConvertUtils.sourceToTarget(dto, Factory.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeStatus(Long id) {
        //改变状态 开启1 关闭0
        Factory entity = baseDao.selectById(id);
        Integer status = 1 - entity.getStatus();
        entity.setStatus(status);
        return updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<FactoryDTO> list() {
        List<FactoryDTO> list = mapper.list();
        return list;
    }

}