package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.basic.dto.ProductFeaturesDTO;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.FeaturesStageProcedureRelationDTO;
import com.cnbm.basic.mapper.FeaturesStageProcedureRelationMapper;
import com.cnbm.basic.entity.FeaturesStageProcedureRelation;
import com.cnbm.basic.service.IFeaturesStageProcedureRelationService;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 过程检验-工艺-检验参数 关系表 
 *
 * @author why
 * @since 2022-09-01
 */
@Service
public class FeaturesStageProcedureRelationServiceImpl extends CrudServiceImpl<FeaturesStageProcedureRelationMapper, FeaturesStageProcedureRelation, FeaturesStageProcedureRelationDTO> implements IFeaturesStageProcedureRelationService {

    @Autowired
    private FeaturesStageProcedureRelationMapper mapper;

    @Override
    public QueryWrapper<FeaturesStageProcedureRelation> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<FeaturesStageProcedureRelation> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<FeaturesStageProcedureRelationDTO> page (Map<String, Object> params){
        IPage<FeaturesStageProcedureRelation> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, FeaturesStageProcedureRelationDTO.class);
    }

    @Override
    public FeaturesStageProcedureRelationDTO get(Long id) {
        FeaturesStageProcedureRelation entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, FeaturesStageProcedureRelationDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(FeaturesStageProcedureRelationDTO dto) {
        FeaturesStageProcedureRelation entity = ConvertUtils.sourceToTarget(dto, FeaturesStageProcedureRelation.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(FeaturesStageProcedureRelationDTO dto) {
        FeaturesStageProcedureRelation entity = ConvertUtils.sourceToTarget(dto, FeaturesStageProcedureRelation.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsert(FeaturesStageProcedureRelationDTO[] lists) {
        for(FeaturesStageProcedureRelationDTO dto:lists){
            ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);
            save(dto);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteByStageProcedure(Map<String, Object> params){

        mapper.deleteByStageProcedure(params);
    }


}