package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cnbm.basic.entity.Platform;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.MachineDTO;
import com.cnbm.basic.mapper.MachineMapper;
import com.cnbm.basic.entity.Machine;
import com.cnbm.basic.service.IMachineService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 机台表
 *
 * @author why
 * @since 2022-07-15
 */
@Service
public class MachineServiceImpl  extends CrudServiceImpl<MachineMapper, Machine, MachineDTO> implements IMachineService {

    @Autowired
    private MachineMapper mapper;

    @Override
    public QueryWrapper<Machine> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");

        QueryWrapper<Machine> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);
        wrapper.eq(ObjectUtils.isNotNull(params.get("status")), "status", params.get("status"));

        return wrapper;
    }

    @Autowired
    private PlatformServiceImpl platformService;

    @Override
    public PageData<MachineDTO> page (Map<String, Object> params) {
        IPage<Machine> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        PageData<MachineDTO> machineDTOPageData = getPageData(page, MachineDTO.class);

        if (!machineDTOPageData.getList().isEmpty()) {

            for (MachineDTO machineDTO : machineDTOPageData.getList()) {
                Platform platform = platformService.selectById(machineDTO.getPlatformId());
                String platformName = null;
                if (platform != null) {
                    platformName = platform.getName();

                }
                machineDTO.setPlatformName(platformName);
            }
        }
        return machineDTOPageData;
    }

    @Override
    public PageData<MachineDTO> page1 (Map<String, Object> params){
//        IPage<Machine> page = baseDao.selectPage(
//                getPage(params, "id", true),
//                getWrapper(params)
//        );
        return mapper.page(params);
    }



    @Override
    public MachineDTO get(Long id) {
        Machine entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, MachineDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(MachineDTO dto) {
        Machine entity = ConvertUtils.sourceToTarget(dto, Machine.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(MachineDTO dto) {
        Machine entity = ConvertUtils.sourceToTarget(dto, Machine.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeStatus(Long id) {
        //改变状态 开启1 关闭0
        Machine entity = baseDao.selectById(id);
        Integer status = 1 - entity.getStatus();
        entity.setStatus(status);
        return updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<MachineDTO> list(Map<String, Object> params) {
        List<MachineDTO> list = mapper.list(params);
        return list;
    }

}