package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.MeasureToolDTO;
import com.cnbm.basic.mapper.MeasureToolMapper;
import com.cnbm.basic.entity.MeasureTool;
import com.cnbm.basic.service.IMeasureToolService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 量具表
 *
 * @author why
 * @since 2022-07-13
 */
@Service
public class MeasureToolServiceImpl extends CrudServiceImpl<MeasureToolMapper, MeasureTool, MeasureToolDTO> implements IMeasureToolService {

    @Override
    public QueryWrapper<MeasureTool> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");

        QueryWrapper<MeasureTool> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);
        wrapper.eq(ObjectUtils.isNotNull(params.get("status")), "status", params.get("status"));

        return wrapper;
    }

    @Override
    public PageData<MeasureToolDTO> page (Map<String, Object> params){
        IPage<MeasureTool> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, MeasureToolDTO.class);
    }

    @Override
    public MeasureToolDTO get(Long id) {
        MeasureTool entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, MeasureToolDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(MeasureToolDTO dto) {
        MeasureTool entity = ConvertUtils.sourceToTarget(dto, MeasureTool.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(MeasureToolDTO dto) {
        MeasureTool entity = ConvertUtils.sourceToTarget(dto, MeasureTool.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeStatus(Long id) {
        //改变状态 开启1 关闭0
        MeasureTool entity = baseDao.selectById(id);
        Integer status = 1 - entity.getStatus();
        entity.setStatus(status);
        return updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Autowired
    private MeasureToolMapper mapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<MeasureToolDTO> list() {
        List<MeasureToolDTO> list = mapper.list();
        return list;
    }

}