package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.basic.dto.ProductWorkingprocedureRelationDTO;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.ProductFactoryRelationDTO;
import com.cnbm.basic.mapper.ProductFactoryRelationMapper;
import com.cnbm.basic.entity.ProductFactoryRelation;
import com.cnbm.basic.service.IProductFactoryRelationService;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Map;

/**
 * 产品-工厂 关系表
 *
 * @author why
 * @since 2022-07-27
 */
@Service
public class ProductFactoryRelationServiceImpl extends CrudServiceImpl<ProductFactoryRelationMapper, ProductFactoryRelation, ProductFactoryRelationDTO> implements IProductFactoryRelationService {

    @Override
    public QueryWrapper<ProductFactoryRelation> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<ProductFactoryRelation> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<ProductFactoryRelationDTO> page (Map<String, Object> params){
        IPage<ProductFactoryRelation> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, ProductFactoryRelationDTO.class);
    }

    @Override
    public ProductFactoryRelationDTO get(Long id) {
        ProductFactoryRelation entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, ProductFactoryRelationDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(ProductFactoryRelationDTO dto) {
        ProductFactoryRelation entity = ConvertUtils.sourceToTarget(dto, ProductFactoryRelation.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(ProductFactoryRelationDTO dto) {
        ProductFactoryRelation entity = ConvertUtils.sourceToTarget(dto, ProductFactoryRelation.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsertFactory(ProductFactoryRelationDTO[] lists) {
        for(ProductFactoryRelationDTO dto:lists){
            ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);
            save(dto);
        }
    }

}