package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.admin.params.IdParam;
import com.cnbm.basic.dto.*;
import com.cnbm.basic.entity.ProductFeaturesHis;
import com.cnbm.basic.entity.ProductWorkingprocedureRelation;
import com.cnbm.basic.entity.WorkingProcedure;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.mapper.ProductFeaturesMapper;
import com.cnbm.basic.entity.ProductFeatures;
import com.cnbm.basic.service.IProductFeaturesService;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 产品特性 表
 *
 * @author why
 * @since 2022-07-15
 */
@Service
public class ProductFeaturesServiceImpl extends CrudServiceImpl<ProductFeaturesMapper, ProductFeatures, ProductFeaturesDTO> implements IProductFeaturesService {

    @Override
    public QueryWrapper<ProductFeatures> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<ProductFeatures> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<ProductFeaturesDTO> page (Map<String, Object> params){
        IPage<ProductFeatures> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, ProductFeaturesDTO.class);
    }

    @Override
    public ProductFeaturesDTO get(Long id) {
        ProductFeatures entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, ProductFeaturesDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(ProductFeaturesDTO dto) {

        ProductFeatures entity = ConvertUtils.sourceToTarget(dto, ProductFeatures.class);
        insert(entity);
    }

    @Autowired
    private ProductFeaturesHisServiceImpl productFeaturesHisServiceImpl;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(ProductFeaturesDTO dto) {
        //若控制图配置为“单值移动极差”，样本量为1
//        if (dto.getControlGraphName()=="单值移动极差"){
//            dto.setSampleSize(1);
//        }
        if(dto.getControlGraphId()!=null){
            String controlGraphName = mapper.getControlGraphNameById(dto.getControlGraphId());
            if((controlGraphName!=null) && (controlGraphName.equals("单值移动极差"))) {
                dto.setSampleSize(1);
            }
        }
        //当产品特性出现update的时候，就会把旧数据 全量备份到这张表
        ProductFeatures oentity = baseDao.selectById(dto.getId());
        ProductFeaturesHisDTO history = ConvertUtils.sourceToTarget(oentity, ProductFeaturesHisDTO.class);
        history.setProductFeaturesId(oentity.getId());
        history.setId(null);
        productFeaturesHisServiceImpl.save(history);
        //更新
        ProductFeatures entity = ConvertUtils.sourceToTarget(dto, ProductFeatures.class);
        updateById(entity);

    }

    @Autowired
    private ProductFeaturesMapper mapper;

//    @Autowired
//    private WorkingProcedureServiceImpl workingProcedureServiceImpl;

    @Override
    public List<ProductFeaturesDTO> getProductFeaturesByProductId(Long id){
        List<ProductFeaturesDTO> list = mapper.getProductFeaturesByProductId(id);
        return list;
        /*
        QueryWrapper<ProductFeatures> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotNull(productId), "product_id", productId);
        List<ProductFeatures> productFeaturesList = mapper.selectList(wrapper);

        List<ProductFeaturesDTO> productFeaturesDTOList = ConvertUtils.sourceToTarget(productFeaturesList,ProductFeaturesDTO.class);
        for(ProductFeaturesDTO dto:productFeaturesDTOList) {
            WorkingProcedure workingProcedure = workingProcedureServiceImpl.selectById(dto.getWorkingProcedureId());
            if (workingProcedure != null) {
                String workingProcedureCode = workingProcedure.getCode();
                String workingProcedureName = workingProcedure.getName();
                dto.setWorkingProcedureCode(workingProcedureCode);
                dto.setWorkingProcedureName(workingProcedureName);
            }

        }
        return productFeaturesDTOList;
         */
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    public List<FeaturesProcedureDTO> getFeaturesByStageProcedure(Map<String, Object> params){
        List<FeaturesProcedureDTO> list = mapper.getFeaturesByStageProcedure(params);
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdate(ProductFeaturesDTO[] lists) {
        for(ProductFeaturesDTO dto:lists){
            //效验数据
            ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);
            update(dto);
        }
    }

    @Override
    public List<ProductFeaturesDTO> getFeaturesList(Map<String, Object> params){
        List<ProductFeaturesDTO> list = mapper.getFeaturesList(params);
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<ProductFeaturesDTO> list() {
        List<ProductFeaturesDTO> list = mapper.list();
        return list;
    }

}