package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;

import com.cnbm.basic.dto.ProductFeaturesDTO;
import com.cnbm.basic.dto.SequenceDTO;
import com.cnbm.basic.dto.WorkingProcedureDTO;
import com.cnbm.basic.entity.WorkingProcedure;
import com.cnbm.basic.mapper.ProductMapper;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.ProductWorkingprocedureRelationDTO;
import com.cnbm.basic.mapper.ProductWorkingprocedureRelationMapper;
import com.cnbm.basic.entity.ProductWorkingprocedureRelation;
import com.cnbm.basic.service.IProductWorkingprocedureRelationService;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 工序 表
 *
 * @author why
 * @since 2022-07-15
 */
@Service
public class ProductWorkingprocedureRelationServiceImpl extends CrudServiceImpl<ProductWorkingprocedureRelationMapper, ProductWorkingprocedureRelation, ProductWorkingprocedureRelationDTO> implements IProductWorkingprocedureRelationService {

    @Autowired
    private ProductWorkingprocedureRelationMapper mapper;

    @Override
    public QueryWrapper<ProductWorkingprocedureRelation> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<ProductWorkingprocedureRelation> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<ProductWorkingprocedureRelationDTO> page (Map<String, Object> params){
        IPage<ProductWorkingprocedureRelation> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, ProductWorkingprocedureRelationDTO.class);
    }

    @Override
    public ProductWorkingprocedureRelationDTO get(Long id) {
        ProductWorkingprocedureRelation entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, ProductWorkingprocedureRelationDTO.class);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(ProductWorkingprocedureRelationDTO dto) {
        //判断数据是否已经绑定过
        QueryWrapper<ProductWorkingprocedureRelation> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotNull(dto.getProductId()),"product_id",dto.getProductId())
                .eq(ObjectUtils.isNotNull(dto.getWorkingProcedureId()),"working_procedure_id",dto.getWorkingProcedureId());
        ProductWorkingprocedureRelation test = mapper.selectOne(wrapper);

        if(Objects.isNull(test)){
            ProductWorkingprocedureRelation entity = ConvertUtils.sourceToTarget(dto, ProductWorkingprocedureRelation.class);
            //设置工序顺序
            List<ProductWorkingprocedureRelationDTO> list = getWorkingprocedureByProductId(dto.getProductId());
            if (list.isEmpty()) {
                entity.setSequence(1);
            } else {
                ProductWorkingprocedureRelationDTO lastDto = list.get(list.size() - 1);
                Integer lastSequence = lastDto.getSequence();
                entity.setSequence(lastSequence + 1);
            }
            insert(entity);
        }
        else{
            dto.setWorkingProcedureId(null);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(ProductWorkingprocedureRelationDTO dto) {
        ProductWorkingprocedureRelation entity = ConvertUtils.sourceToTarget(dto, ProductWorkingprocedureRelation.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    /*
    @Autowired
    private WorkingProcedureServiceImpl workingProcedureServiceImpl;

    @Override
    public List<ProductWorkingprocedureRelationDTO> getWorkingprocedureByProductId(Long productId){
        QueryWrapper<ProductWorkingprocedureRelation> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotNull(productId), "product_id", productId);
        List<ProductWorkingprocedureRelation> relationList = baseDao.selectList(wrapper);
        List<ProductWorkingprocedureRelationDTO> dtoList = ConvertUtils.sourceToTarget(relationList, ProductWorkingprocedureRelationDTO.class);
        //设置工序名称
        for (ProductWorkingprocedureRelationDTO dto:dtoList){
            WorkingProcedure workingProcedure = workingProcedureServiceImpl.selectById(dto.getWorkingProcedureId());
            String workingProcedureName = null;
            String workingProcedureCode = null;
            if(workingProcedure!=null){
                workingProcedureName = workingProcedure.getName();
                workingProcedureCode = workingProcedure.getCode();
            }
            dto.setWorkingProcedureName(workingProcedureName);
            dto.setWorkingProcedureCode(workingProcedureCode);
        }
        return dtoList;
    }
     */

    @Override
    public List<ProductWorkingprocedureRelationDTO> getWorkingprocedureByProductId(Long productId){
        List<ProductWorkingprocedureRelationDTO> list= mapper.getWorkingprocedureByProductId(productId);
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsertWorkingprocedure(ProductWorkingprocedureRelationDTO[] lists) {
        for(ProductWorkingprocedureRelationDTO dto:lists){
            ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);
            save(dto);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void sequenceUp(SequenceDTO sequenceDTO)
    {
        Integer index = sequenceDTO.getIndex();
        Long productId = sequenceDTO.getProductId();
        //排序为最前不能再上移
        if(index!= 1){
            List<ProductWorkingprocedureRelationDTO> list = getWorkingprocedureByProductId(productId);
            ProductWorkingprocedureRelationDTO dto = list.get(index-1);
            ProductWorkingprocedureRelationDTO beforeDto = list.get(index-2);

            //交换顺序
            Integer dtoSequence = dto.getSequence();
            Integer beforeDtoSequence = beforeDto.getSequence();

            dto.setSequence(beforeDtoSequence);
            beforeDto.setSequence(dtoSequence);

            update(dto);
            update(beforeDto);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void sequenceDown(SequenceDTO sequenceDTO)
    {
        Integer index = sequenceDTO.getIndex();
        Long productId = sequenceDTO.getProductId();
        List<ProductWorkingprocedureRelationDTO> list = getWorkingprocedureByProductId(productId);
        //排序为最后不能再下移
        if(index !=list.size()){
            ProductWorkingprocedureRelationDTO dto = list.get(index-1);
            ProductWorkingprocedureRelationDTO afterDto = list.get(index);

            //交换顺序
            Integer dtoSequence = dto.getSequence();
            Integer afterDtoSequence = afterDto.getSequence();

            dto.setSequence(afterDtoSequence);
            afterDto.setSequence(dtoSequence);

            update(dto);
            update(afterDto);
        }
    }
}