package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.ShiftDTO;
import com.cnbm.basic.mapper.ShiftMapper;
import com.cnbm.basic.entity.Shift;
import com.cnbm.basic.service.IShiftService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 班次 表
 *
 * @author why
 * @since 2022-07-13
 */
@Service
public class ShiftServiceImpl extends CrudServiceImpl<ShiftMapper, Shift, ShiftDTO> implements IShiftService {

    @Autowired
    private ShiftMapper mapper;

    @Override
    public QueryWrapper<Shift> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<Shift> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<ShiftDTO> page (Map<String, Object> params){

        IPage<Shift> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, ShiftDTO.class);
    }

    @Override
    public ShiftDTO get(Long id) {
        Shift entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, ShiftDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(ShiftDTO dto) {
        Shift entity = ConvertUtils.sourceToTarget(dto, Shift.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(ShiftDTO dto) {
        Shift entity = ConvertUtils.sourceToTarget(dto, Shift.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<ShiftDTO> list() {
        List<ShiftDTO> list = mapper.list();
        return list;
    }

}