package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.TeamDTO;
import com.cnbm.basic.mapper.TeamMapper;
import com.cnbm.basic.entity.Team;
import com.cnbm.basic.service.ITeamService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 班组 表
 *
 * @author why
 * @since 2022-07-13
 */
@Service
public class TeamServiceImpl extends CrudServiceImpl<TeamMapper, Team, TeamDTO> implements ITeamService {

    @Autowired
    private TeamMapper mapper;

    @Override
    public QueryWrapper<Team> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<Team> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<TeamDTO> page (Map<String, Object> params){
        IPage<Team> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, TeamDTO.class);
    }

    @Override
    public TeamDTO get(Long id) {
        Team entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, TeamDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(TeamDTO dto) {
        Team entity = ConvertUtils.sourceToTarget(dto, Team.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(TeamDTO dto) {
        Team entity = ConvertUtils.sourceToTarget(dto, Team.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<TeamDTO> list() {
        List<TeamDTO> list = mapper.list();
        return list;
    }

}