package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.basic.dto.UnitDTO;
import com.cnbm.basic.entity.Machine;
import com.cnbm.basic.entity.ProductWorkingprocedureRelation;
import com.cnbm.basic.entity.WorkingProcedureType;
import com.cnbm.basic.mapper.ProductWorkingprocedureRelationMapper;
import com.cnbm.basic.mapper.UnitMapper;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.WorkingProcedureDTO;
import com.cnbm.basic.mapper.WorkingProcedureMapper;
import com.cnbm.basic.entity.WorkingProcedure;
import com.cnbm.basic.service.IWorkingProcedureService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 工序 表
 *
 * @author why
 * @since 2022-07-15
 */
@Service
public class WorkingProcedureServiceImpl extends CrudServiceImpl<WorkingProcedureMapper, WorkingProcedure, WorkingProcedureDTO> implements IWorkingProcedureService {

    @Override
    public QueryWrapper<WorkingProcedure> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");

        QueryWrapper<WorkingProcedure> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);
        wrapper.eq(ObjectUtils.isNotNull(params.get("status")), "status", params.get("status"));

        return wrapper;
    }

    @Autowired
    private MachineServiceImpl machineServiceImpl;

    @Autowired
    private WorkingProcedureTypeServiceImpl workingProcedureTypeServiceImpl;

    @Override
    public PageData<WorkingProcedureDTO> page (Map<String, Object> params){
        IPage<WorkingProcedure> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        PageData<WorkingProcedureDTO> workingProcedureDTOPageData = getPageData(page, WorkingProcedureDTO.class);
        if(!workingProcedureDTOPageData.getList().isEmpty()){
            for(WorkingProcedureDTO workingProcedureDTO:workingProcedureDTOPageData.getList()){
                //工序类型 id,关联working_procedure_type表
                WorkingProcedureType workingProcedureType = workingProcedureTypeServiceImpl.selectById(workingProcedureDTO.getWorkingProcedureTypeId());
                String workingProcedureTypeName = null;
                if(workingProcedureType!=null){
                    workingProcedureTypeName = workingProcedureType.getName();
                }
                workingProcedureDTO.setWorkingProcedureTypeName(workingProcedureTypeName);

                //机台(也就是设备)，这个工序对应的设备，可能有一个或者多个，如果多个用逗号隔开，"id1,id2,...
                //String machineNameList = null;
                if(StringUtils.isNotBlank(workingProcedureDTO.getMachineId())){
                    String[] machineIdList = workingProcedureDTO.getMachineId().split(",");
                    ArrayList<String> machineList = new ArrayList<>();
                    for(String machineId:machineIdList){
                        //QueryWrapper<Machine> machineQueryWrapper = new QueryWrapper<>();
                        //machineQueryWrapper.eq(StringUtils.isNotBlank(machineId), "id", machineId);
                        //String machineName = null;
                        Machine machine = machineServiceImpl.selectById(machineId);
                        if(machine!=null&&machine.getName()!=null) {
                            machineList.add(machine.getName());
                        }
                    }
                    String machineNameList = String.join(",", machineList);
                    workingProcedureDTO.setMachineName(machineNameList);
                }

            }
        }
        return workingProcedureDTOPageData;
    }

    @Override
    public WorkingProcedureDTO get(Long id) {
        WorkingProcedure entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WorkingProcedureDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WorkingProcedureDTO dto) {
        WorkingProcedure entity = ConvertUtils.sourceToTarget(dto, WorkingProcedure.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WorkingProcedureDTO dto) {
        WorkingProcedure entity = ConvertUtils.sourceToTarget(dto, WorkingProcedure.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeStatus(Long id) {
        //改变状态 开启1 关闭0
        WorkingProcedure entity = baseDao.selectById(id);
        Integer status = 1 - entity.getStatus();
        entity.setStatus(status);
        return updateById(entity);
    }

    @Autowired
    private ProductWorkingprocedureRelationMapper mapper;

    @Override
    public List<WorkingProcedureDTO> getWorkingProcedureByProductId(Long productId){
        QueryWrapper<ProductWorkingprocedureRelation> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotNull(productId), "product_id", productId);
        List<ProductWorkingprocedureRelation> productWorkingprocedureRelationList = mapper.selectList(wrapper);

        List<WorkingProcedureDTO> workingProcedureDTOList = new ArrayList<>();
        for (ProductWorkingprocedureRelation productWorkingprocedureRelation:productWorkingprocedureRelationList){
            WorkingProcedureDTO workingProcedureDTO = get(productWorkingprocedureRelation.getWorkingProcedureId());
            workingProcedureDTOList.add(workingProcedureDTO);
        }
        return workingProcedureDTOList;
    }


    @Autowired
    private WorkingProcedureMapper workingProcedureMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WorkingProcedureDTO> list() {
        List<WorkingProcedureDTO> list =  workingProcedureMapper.list();
        return list;
    }

}