package com.cnbm.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.basic.dto.WorkingProcedureTypeDTO;
import com.cnbm.basic.mapper.WorkingProcedureTypeMapper;
import com.cnbm.basic.entity.WorkingProcedureType;
import com.cnbm.basic.service.IWorkingProcedureTypeService;
import com.cnbm.common.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Map;

/**
 * 工序类型表 表
 *
 * @author why
 * @since 2022-07-15
 */
@Service
public class WorkingProcedureTypeServiceImpl extends CrudServiceImpl<WorkingProcedureTypeMapper, WorkingProcedureType, WorkingProcedureTypeDTO> implements IWorkingProcedureTypeService {

    @Override
    public QueryWrapper<WorkingProcedureType> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");

        QueryWrapper<WorkingProcedureType> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);
        wrapper.eq(ObjectUtils.isNotNull(params.get("status")), "status", params.get("status"));

        return wrapper;
    }

    @Override
    public PageData<WorkingProcedureTypeDTO> page (Map<String, Object> params){
        IPage<WorkingProcedureType> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, WorkingProcedureTypeDTO.class);
    }

    @Override
    public WorkingProcedureTypeDTO get(Long id) {
        WorkingProcedureType entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WorkingProcedureTypeDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WorkingProcedureTypeDTO dto) {
        WorkingProcedureType entity = ConvertUtils.sourceToTarget(dto, WorkingProcedureType.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WorkingProcedureTypeDTO dto) {
        WorkingProcedureType entity = ConvertUtils.sourceToTarget(dto, WorkingProcedureType.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeStatus(Long id) {
        //改变状态 开启1 关闭0
        WorkingProcedureType entity = baseDao.selectById(id);
        Integer status = 1 - entity.getStatus();
        entity.setStatus(status);
        return updateById(entity);
    }

}