package com.cnbm.processInspection.graphAnalyzed.forCount.np;

import com.cnbm.basic.entity.ProductFeatures;
import com.cnbm.common.spc.util.DataUtils;
import com.cnbm.influx.config.InfluxClient;
import com.cnbm.influx.constant.Constant;
import com.cnbm.influx.param.QueryDataGroupByTimeParam;
import com.cnbm.influx.param.Tag;
import com.cnbm.qualityPlanning.entity.*;
import com.influxdb.query.FluxRecord;
import com.influxdb.query.FluxTable;
import lombok.Data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Desc: "均值标准差 控制图 , 计算类"
 * @Author: caixiang
 * @DATE: 2022/7/20 14:26
 * 使用方式：① 先new MeanStandardDeviationGraph 对象 ；② 再initialData 初始化数据；③ 再get 控制限
 *
 * 步骤：
 *      ① 先读mysql表，查询 product_features 表，先读到 sample_size（样本量）
 *      ② 再依据 influx.argName == mysql.product_feature.name && 时间段 查询所有的 参数数据
 *      ③ 拿到参数数据后，分组 整合成List<Point>,
 *          计算控制限
 *          计算 母体 的 \sigma 、 bar{x} 。。。
 *          计算CPK 、CPU 、CPL这些
 *      ④ 如果配置了判读方案，还要 调用 StatisticalControlledTest Function 检验。
 *      ⑤
 */
@Data
public class NPGraph {


    //计数型，不能用判读方案校验，，因为 当每个样本n不同，控制限 都不一定相同。
//    private List<InterpretationListArgForCount> interpretationScheme;
    private String argName;

    private List<NPPoint> list;

    private Double pbar;

    private SpecificationLimit specificationLimit;

    public NPGraph(ProductFeatures productFeatures) throws Exception {
        this.argName = productFeatures.getName();
        list = new ArrayList<>();
        this.specificationLimit = new SpecificationLimit(
                productFeatures.getUsl()==null?null:productFeatures.getUsl(),
                productFeatures.getSl()==null?null:productFeatures.getSl(),
                productFeatures.getLsl()==null?null:productFeatures.getLsl()
                );
    }

    private Double[] toDoubleArray(Object[] o){
        Double[] res= new Double[o.length];
        for(int i=0;i<o.length;i++){
            res[i] = (Double) o[i];
        }
        return res;
    }

    private Double computePbar(List<FluxTable> query){
        Double totalFailNum = (double)0;
        Double totalN = (double)0;

        for (FluxTable fluxTable : query) {
            List<FluxRecord> records = fluxTable.getRecords();

            Integer failNum = 0;
            Integer n = records.size();
            for (FluxRecord fluxRecord : records) {
                //因为 传进去的就是Double 类型，所以取出来，自然而然就是Double
                Double value = Double.parseDouble(fluxRecord.getValueByKey("_value").toString());
                if(value.equals((double) 0)){
                    failNum+=1;
                }
            }
            totalFailNum =totalFailNum + (double)failNum;
            totalN = totalN + (double)n;
        }

        return totalFailNum/totalN;
    }

    /**
     * name : 初始化数据函数
     * desc : 从influxdb 里面读取数据，然后 加工处理成 我需要的
     * 步骤：
     *      ①
     * */
    public void initialDate(QueryDataGroupByTimeParam queryDataParam){
        queryDataParam.setBucket(Constant.bucket);
        List<String> dropNames = new ArrayList<>();
        dropNames.add("transationId");
        dropNames.add("inspectionSheetId");
        //dropNames.add("batchNum");
        queryDataParam.setDropedTagNames(dropNames);

        queryDataParam.setTags(Arrays.asList(new Tag("argName",argName)));
        List<FluxTable> query = InfluxClient.Client.queryGroupByTime(queryDataParam);
        //1. 先从fluxdb 里面提取原始数据
        List<Double> originData = new ArrayList<>();
        //计算p bar
        this.pbar = computePbar(query);

        //2.计算各项式
        for(int i=0 ;i<query.size();i++){
            List<FluxRecord> records = query.get(i).getRecords();
            Integer failNum = 0;
            Integer n = records.size();
            String name = DataUtils.splitToNeed(records.get(0).getTime().toString(),queryDataParam.getTimeType());
            for (FluxRecord fluxRecord : records) {
                //因为 传进去的就是Double 类型，所以取出来，自然而然就是Double
                Double value = Double.parseDouble(fluxRecord.getValueByKey("_value").toString());
                if(value.equals((double) 0)){
                    failNum+=1;
                }
            }

            list.add(new NPPoint(
                    getCL((double)n),
                    i,
                    failNum,
                    name
                    ));
        }
    }




    /**
     * desc: get Xbar控制图 的控制限
     * 注意：此函数 要在 initialDate()函数执行之后
     * */
    public ControlLimitDetail getCL(Double n){
        Double npbar = n * this.pbar;
        Double mul = 3 * Math.sqrt(npbar*(1-this.pbar));
        Double lcl = (npbar-mul)<0?0:(npbar-mul);
        return new ControlLimitDetail(
                npbar + mul,
                npbar,
                lcl
        );
    }
}
