package com.cnbm.processInspection.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.basic.dto.ProductFeaturesDTO;
import com.cnbm.basic.dto.WorkingProcedureDTO;
import com.cnbm.basic.entity.*;
import com.cnbm.basic.service.impl.*;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.spc.util.DataUtils;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.influx.config.InfluxClient;
import com.cnbm.influx.param.PageInfo;
import com.cnbm.influx.param.QueryDataParam;
import com.cnbm.influx.param.Range;

import com.cnbm.influx.param.Tag;
import com.cnbm.influx.template.Event;
import com.cnbm.influx.template.Sample;
import com.cnbm.influx.template.SampleParam;
import com.cnbm.processInspection.dto.InspectionSampleDTO;
import com.cnbm.processInspection.dto.InspectionSampleDTO2;
import com.cnbm.processInspection.dto.InspectionSampleDTO3;
import com.cnbm.processInspection.dto.InspectionSheetDTO;
import com.cnbm.processInspection.entity.InspectionSheet;
import com.cnbm.processInspection.mapper.InspectionSheetMapper;
import com.cnbm.processInspection.service.IInspectionSheetService;
import com.influxdb.query.FluxRecord;
import com.influxdb.query.FluxTable;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 检验单 表
 *
 * @author why
 * @since 2022-08-17
 */
@Service
public class InspectionSheetServiceImpl extends CrudServiceImpl<InspectionSheetMapper, InspectionSheet, InspectionSheetDTO> implements IInspectionSheetService {

    @Autowired
    private ProductServiceImpl productService;

    @Autowired
    private WorkingProcedureServiceImpl workingProcedureService;

    @Autowired
    private MachineServiceImpl machineService;

    @Autowired
    private ShiftServiceImpl shiftService;

    @Autowired
    private ProductFeaturesServiceImpl productFeaturesService;


    @Override
    public QueryWrapper<InspectionSheet> getWrapper(Map<String, Object> params) {
        Long id = (Long) params.get("id");
        String inspectionSite = (String) params.get("inspectionSite");
        String orderNumber = (String) params.get("orderNumber");
        String batchNumber = (String) params.get("batchNumber");

        QueryWrapper<InspectionSheet> wrapper = new QueryWrapper<>();
        wrapper.like(ObjectUtils.isNotNull(id), "id", id);
        wrapper.like(StringUtils.isNotBlank(inspectionSite), "inspection_stage", inspectionSite);
        wrapper.like(StringUtils.isNotBlank(orderNumber), "order_number", orderNumber);
        wrapper.like(StringUtils.isNotBlank(batchNumber), "batch_number", batchNumber);
        wrapper.eq(ObjectUtils.isNotNull(params.get("productId")), "product_id", params.get("productId"));
        wrapper.eq(ObjectUtils.isNotNull(params.get("workingProcedureId")), "working_procedure_id", params.get("workingProcedureId"));
        wrapper.eq(ObjectUtils.isNotNull(params.get("machineId")), "machine_id", params.get("machineId"));
        return wrapper;
    }

    @Override
    public PageData<InspectionSheetDTO> page(Map<String, Object> params) {
        IPage<InspectionSheet> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        PageData<InspectionSheetDTO> inspectionSheetDTOPageData = getPageData(page, InspectionSheetDTO.class);

        if (!inspectionSheetDTOPageData.getList().isEmpty()) {
            for (InspectionSheetDTO dto : inspectionSheetDTOPageData.getList()) {
                Product product = productService.selectById(dto.getProductId());
                WorkingProcedure workingProcedure = workingProcedureService.selectById(dto.getWorkingProcedureId());
                Machine machine = machineService.selectById(dto.getMachineId());
                Shift shift = shiftService.selectById(dto.getShiftId());

                String productName = null;
                String workingProcedureName = null;
                String machineName = null;
                String shiftName = null;

                if (product != null) {
                    productName = product.getName();
                }
                if (workingProcedure != null) {
                    workingProcedureName = workingProcedure.getName();
                }
                if (machine != null) {
                    machineName = machine.getName();
                }
                if (shift != null) {
                    shiftName = shift.getName();
                }

                dto.setProductName(productName);
                dto.setWorkingProcedureName(workingProcedureName);
                dto.setMachineName(machineName);
                dto.setShiftName(shiftName);
            }
        }
        return inspectionSheetDTOPageData;
    }

    @Override
    public InspectionSheetDTO get(Long id) {
        InspectionSheet entity = baseDao.selectById(id);
        InspectionSheetDTO dto = ConvertUtils.sourceToTarget(entity, InspectionSheetDTO.class);

        if(dto.getWorkingProcedureId()!=null) {
            WorkingProcedure workingProcedure = workingProcedureService.selectById(dto.getWorkingProcedureId());
            if (workingProcedure != null) {
                dto.setWorkingProcedureName(workingProcedure.getName());
            }
        }
        return dto;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public InspectionSheet saveSheet(InspectionSheetDTO dto) {
        // 验证是否有检验参数
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("productId", dto.getProductId());
        params.put("workingProcedureId", dto.getWorkingProcedureId());
        params.put("inspectionStage", dto.getInspectionStage());
        List<ProductFeaturesDTO> inspectionSheetFeaturesList = getInspectionSheetFeaturesList(params);
        if (inspectionSheetFeaturesList != null && inspectionSheetFeaturesList.size() != 0) {
            //分组样本数=样本大小=检验特性分组数的最大值
            Integer numbersOfSamples = inspectionSheetFeaturesList.stream().max(Comparator.comparing(ProductFeaturesDTO::getSampleSize)).get().getSampleSize();
            if (numbersOfSamples != null) {
                dto.setNumberOfGroupedSamples(numbersOfSamples);
                dto.setNumberOfSamples(numbersOfSamples);
            }
        }
        /*
        if (inspectionSheetFeaturesList == null || inspectionSheetFeaturesList.size() == 0) {
            throw new RuntimeException("没有发现检验参数");
        } else {
            //分组样本数=样本大小=检验特性分组数的最大值
            Integer numbersOfSamples = inspectionSheetFeaturesList.stream().max(Comparator.comparing(ProductFeaturesDTO::getSampleSize)).get().getSampleSize();
            if (numbersOfSamples != null) {
                dto.setNumberOfGroupedSamples(numbersOfSamples);
                dto.setNumberOfSamples(numbersOfSamples);
            }
        }
        */
        //InspectionSheet entity = ConvertUtils.sourceToTarget(dto, InspectionSheet.class);
        InspectionSheet entity =  new InspectionSheet();
        BeanUtils.copyProperties(dto, entity);
        return entity;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(InspectionSheetDTO dto) {
        InspectionSheet entity = ConvertUtils.sourceToTarget(dto, InspectionSheet.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Resource
    private InspectionSheetMapper mapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<InspectionSheetDTO> list(Map<String, Object> params) {
        List<InspectionSheetDTO> list = mapper.list(params);
        return list;
    }

    @Override
    public List<InspectionSampleDTO> getFluxParamList(Long id) {
        List<Event> eventList = new ArrayList<>();
        InspectionSheetDTO dto = get(id);
        if (dto != null && dto.getWorkingProcedureName()!=null) {
            QueryDataParam queryDataParam = new QueryDataParam();
            String inspectionSheetId = id.toString();
            String workingProcedureName = dto.getWorkingProcedureName();
            queryDataParam.setMeasurement(workingProcedureName);
            //queryDataParam.setInspectionSheetId(inspectionSheetId);
            queryDataParam.setBucket("qgs-bucket");
            queryDataParam.setTags(Arrays.asList(new Tag("inspectionSheetId", inspectionSheetId)));
            List<String> dropNames = new ArrayList<>();
            dropNames.add("transationId");
            queryDataParam.setDropedTagNames(dropNames);
            queryDataParam.setRange(new Range(DataUtils.getBeforeDate(999).toInstant(), DataUtils.getAfterDate(999).toInstant()));
            queryDataParam.setPageInfo(new PageInfo(1, 10));

            List<FluxTable> fluxTableList = InfluxClient.Client.query(queryDataParam);
            for (FluxTable fluxTable : fluxTableList) {
                List<FluxRecord> records = fluxTable.getRecords(); // 获取实际的记录操作
                for (FluxRecord fluxRecord : records) {
                    Instant time = fluxRecord.getTime();
                    String argName = (String) fluxRecord.getValueByKey("argName");
                    String argValue = (String)fluxRecord.getValueByKey("_value");
                    String sampleNo = (String) fluxRecord.getValueByKey("sampleNumber");
                    eventList.add(newEvent(time, inspectionSheetId, argName, argValue, sampleNo));
                }
            }
        }
        List<InspectionSampleDTO> list = new ArrayList<>();
        if (eventList != null || eventList.size() > 0) {
            //根据样本号分组
            Map<String, List<Event>> map = eventList.stream().collect(Collectors.groupingBy(Event::getSampleNumber));
            for (Map.Entry<String, List<Event>> entry : map.entrySet()) {
                String sampleNumber = entry.getKey();
                List<Event> events = entry.getValue();

                InspectionSampleDTO sampleDTO = new InspectionSampleDTO();
                sampleDTO.setSampleNumber(sampleNumber);
                //sampleDTO.setSampleTime(LocalDateTime.ofInstant(events.get(0).getTime(), ZoneId.systemDefault()));
                sampleDTO.setSampleTime(events.get(0).getTime());
                sampleDTO.setBatchNum(dto.getBatchNumber());
                sampleDTO.setInspectionSheetId(events.get(0).getInspectionSheetId());

                JSONObject jsonObject = new JSONObject();
                for(Event event:events){
                    jsonObject.put(event.getArgName(),event.getArgValue());
                }
                sampleDTO.setJsonData(jsonObject.toString());
                //System.out.println(entry);
                list.add(sampleDTO);
            }
        }
        //返回先按时间排序 再按样本号排序
        List<InspectionSampleDTO> sortedList = list.stream()
                .sorted(Comparator.comparing(InspectionSampleDTO::getSampleTime).thenComparing(InspectionSampleDTO::getSampleNumber))
                .collect(Collectors.toList());
        return sortedList;
    }

    @Override
    public void saveFluxParamList(InspectionSampleDTO[] lists){
        for (InspectionSampleDTO dto : lists) {
            String workingProcedureName = dto.getWorkingProcedureName();
            String inspectionSheetId = dto.getInspectionSheetId();
//            String batchNum = dto.getBatchNum();
            String sampleNumber = dto.getSampleNumber();
            String jsonData = dto.getJsonData();
            JSONObject json = JSON.parseObject(jsonData);
//            LocalDateTime sampleTime = dto.getSampleTime();
//            Instant eventTime = sampleTime.toInstant(ZoneOffset.UTC);
            Instant eventTime = dto.getSampleTime();
            List<Event> list = new ArrayList<>();
            for (Map.Entry entry : json.entrySet()) {
                String key = entry.getKey().toString();
                String v = entry.getValue().toString();
//                Double v = null;
//                try {
//                    v= Double.valueOf(entry.getValue().toString());
//                }catch (Exception e){
//                }
                list.add(newEvent(eventTime, inspectionSheetId, key, v, sampleNumber));
            }
            InfluxClient.Client.batchInsert(list, workingProcedureName);
        }
//        //样本数据更新后 计算检验单缺陷数不良数
//        System.out.println("--------------------------------------");
//        calculate(Long.valueOf(lists[0].getInspectionSheetId()));
    }

    @Override
    public void saveFluxParamList2(InspectionSampleDTO2[] lists){
        for (InspectionSampleDTO2 dto : lists) {
            String workingProcedureName = dto.getWorkingProcedureName();
            InfluxClient.Client.batchInsert(dto.getEvents(), workingProcedureName);
        }
    }

    private Event newEvent(Instant time, String inspectionSheetId, String argName, String argValue, String sampleNo) {
        Event event = new Event();
        event.setInspectionSheetId(inspectionSheetId);
        event.setTime(time);
//        event.setBatchNum(batchNum);
        event.setSampleNumber(sampleNo);
        event.setArgName(argName);
        event.setArgValue(argValue);
//        if (Objects.equals(argValue, "")){
//            event.setArgValue("");
//        }else if (argValue != null) {
//            event.setArgValue(argValue.toString());
//        }
        return event;
    }

//    @Override
//    public Integer getNumberOfSamples(Map<String, Object> params) {
//        return mapper.getNumberOfSamples(params);
//    }

    @Override
    public List<ProductFeaturesDTO> getInspectionSheetFeaturesList(Map<String, Object> params) {
        return mapper.getInspectionSheetFeaturesList(params);
    }

    //计算并更新检验单缺陷数不良数
    @Override
    public void calculate(Long inspectionSheetId) {
        List<InspectionSampleDTO3> sampleDTOList = getFluxParamList3(inspectionSheetId);
        if (sampleDTOList != null || sampleDTOList.size() > 0) {
            //缺陷
            Integer numberOfDefects = 0;
            //不良
            Integer defectiveQuantity = 0;
            //获取的产品特性参照值
            Long productId = get(inspectionSheetId).getProductId();
            List<ProductFeaturesDTO> featuresList = productFeaturesService.getProductFeaturesByProductId(productId);
            Map<String, ProductFeaturesDTO> featureMap = featuresList.stream().collect(Collectors.groupingBy(ProductFeaturesDTO::getName, Collectors.collectingAndThen(Collectors.toList(), f -> f.get(0))));
            //循环每个样本组
            for (InspectionSampleDTO3 sampleDTO : sampleDTOList) {
                //该样本的缺陷数
                Integer eventDefects = 0;
                //该样本的属性列表
                List<SampleParam> sampleParamList = sampleDTO.getSampleParamList();
                for(SampleParam sampleParam : sampleParamList){
                    String argName = sampleParam.getArgName();
                    String argValue = sampleParam.getArgValue();
                    ProductFeaturesDTO feature = featureMap.get(argName);
                    //属性值非空 对应产品特性非空
                    if(!Objects.equals(argValue, "") && argValue != null && feature!=null){
                        //1 计量型；2 计数型
                        if (feature.getType() == 1) {
                            Double featureValue = Double.valueOf(argValue);
                            if (featureValue > feature.getUsl() || featureValue < feature.getLsl()) {
                                eventDefects = eventDefects + 1;
                            }
                        }
                        if (feature.getType() == 2) {
                            Double featureValue = Double.valueOf(argValue);
                            if (featureValue == 1) {
                                eventDefects = eventDefects + 1;
                            }
                        }
                    }
                }
                //该样本是否有缺陷
                if (eventDefects != 0) {
                    numberOfDefects = numberOfDefects + eventDefects;
                    defectiveQuantity = defectiveQuantity + 1;
                }
            }
            InspectionSheetDTO updateDto = new InspectionSheetDTO();
            updateDto.setId(inspectionSheetId);
            updateDto.setNumberOfDefects(numberOfDefects);
            updateDto.setDefectiveQuantity(defectiveQuantity);
            update(updateDto);
        }
    }

    @Override
    public void saveFluxParamList3(InspectionSampleDTO3[] lists){
        for (InspectionSampleDTO3 dto : lists) {
            String inspectionSheetId = dto.getInspectionSheetId();
            Instant eventTime = dto.getSampleTime();
            String sampleNumber = dto.getSampleNumber();

            List<SampleParam> sampleParamList = dto.getSampleParamList();
            for(SampleParam sampleParam : sampleParamList){
                String workingProcedureName = sampleParam.getWorkingProcedureName();
                String paramName = sampleParam.getArgName();
                String paramValue = sampleParam.getArgValue();
                Event event = newEvent(eventTime, inspectionSheetId, paramName, paramValue, sampleNumber);
                InfluxClient.Client.insert(event, workingProcedureName);
            }
        }
    }

    @Override
    public List<InspectionSampleDTO3> getFluxParamList3(Long id) {
        List<Sample> sampleList = new ArrayList<>();
        //获取influx查询参数
        InspectionSheetDTO dto = get(id);
        if(dto!=null){
            //检验单-单一工序
            if(dto.getWorkingProcedureName()!=null){
                String workingProcedureName = dto.getWorkingProcedureName();
                sampleList = getEventList(id.toString(),workingProcedureName);

            }
            //检验单-所有工序
            else {
                List<WorkingProcedureDTO> workingProcedureDTOList = workingProcedureService.getWorkingProcedureByProductId(dto.getProductId());
                for(WorkingProcedureDTO workingProcedureDTO:workingProcedureDTOList){
                    List<Sample> sampleList1 = getEventList(id.toString(),workingProcedureDTO.getName());
                    sampleList.addAll(sampleList1);
                }
            }
        }
        List<InspectionSampleDTO3> list = new ArrayList<>();
        if (sampleList != null || sampleList.size() > 0) {
            //根据样本号分组
            Map<String, List<Sample>> map = sampleList.stream().collect(Collectors.groupingBy(Sample::getSampleNumber));
            for (Map.Entry<String, List<Sample>> entry : map.entrySet()) {
                String sampleNumber = entry.getKey();
                List<Sample> samples = entry.getValue();
                InspectionSampleDTO3 sampleDTO = new InspectionSampleDTO3();
                sampleDTO.setSampleNumber(sampleNumber);
                sampleDTO.setSampleTime(samples.get(0).getSampleTime());
                sampleDTO.setBatchNum(dto.getBatchNumber());
                sampleDTO.setInspectionSheetId(samples.get(0).getInspectionSheetId());
                List<SampleParam> sampleParamList = new ArrayList<>();
                for(Sample sample:samples){
                    SampleParam sampleParam = new SampleParam();
                    sampleParam.setArgName(sample.getArgName());
                    sampleParam.setArgValue(sample.getArgValue());
                    sampleParam.setWorkingProcedureName(sample.getWorkingProcedureName());
                    sampleParamList.add(sampleParam);
                }
                sampleDTO.setSampleParamList(sampleParamList);
                list.add(sampleDTO);
            }
        }
        //返回先按时间排序 再按样本号排序
        List<InspectionSampleDTO3> sortedList = list.stream()
                .sorted(Comparator.comparing(InspectionSampleDTO3::getSampleTime).thenComparing(InspectionSampleDTO3::getSampleNumber))
                .collect(Collectors.toList());
        return sortedList;
    }

    private List<Sample> getEventList(String inspectionSheetId, String workingProcedureName) {
        List<Sample> sampleList = new ArrayList<>();
        QueryDataParam queryDataParam = new QueryDataParam();
        queryDataParam.setMeasurement(workingProcedureName);
        queryDataParam.setBucket("qgs-bucket");
        queryDataParam.setTags(Arrays.asList(new Tag("inspectionSheetId", inspectionSheetId.toString())));
        List<String> dropNames = new ArrayList<>();
        dropNames.add("transationId");
        queryDataParam.setDropedTagNames(dropNames);
        queryDataParam.setRange(new Range(DataUtils.getBeforeDate(999).toInstant(), DataUtils.getAfterDate(999).toInstant()));
        queryDataParam.setPageInfo(new PageInfo(1, 10));
        List<FluxTable> fluxTableList = InfluxClient.Client.query(queryDataParam);
        for (FluxTable fluxTable : fluxTableList) {
            List<FluxRecord> records = fluxTable.getRecords(); // 获取实际的记录操作
            for (FluxRecord fluxRecord : records) {
                Instant time = fluxRecord.getTime();
                String argName = (String) fluxRecord.getValueByKey("argName");
                String argValue = (String) fluxRecord.getValueByKey("_value");
                String sampleNo = (String) fluxRecord.getValueByKey("sampleNumber");
                sampleList.add(newSample(time, inspectionSheetId, workingProcedureName, argName, argValue, sampleNo));
            }
        }
        return sampleList;
    }

    private Sample newSample(Instant time, String inspectionSheetId, String workingProcedureName, String argName, String argValue, String sampleNo) {
        Sample sample = new Sample();
        sample.setInspectionSheetId(inspectionSheetId);
        sample.setWorkingProcedureName(workingProcedureName);
        sample.setSampleTime(time);
        sample.setSampleNumber(sampleNo);
        sample.setArgName(argName);
        sample.setArgValue(argValue);
        return sample;
    }

}