package com.cnbm.qualityPlanning.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.qualityPlanning.dto.InterpretationSchemeDTO;
import com.cnbm.qualityPlanning.excel.InterpretationSchemeExcel;
import com.cnbm.qualityPlanning.service.IInterpretationSchemeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 判读方案 表  前端控制器
 *
 * @author why
 * @since  2022-08-15
 */
@RestController
@RequestMapping("/qualityPlanning/interpretationScheme")
@Api(tags="判读方案 表")
public class InterpretationSchemeController {
    @Autowired
    private IInterpretationSchemeService interpretationSchemeService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "name", value = "名称", paramType = "query", dataTypeClass = String.class),
        @ApiImplicitParam(name = "code", value = "编码", paramType = "query", dataTypeClass = String.class)
    })
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:interpretationScheme:page')")
    public Result<PageData<InterpretationSchemeDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<InterpretationSchemeDTO> page = interpretationSchemeService.page(params);

        return new Result<PageData<InterpretationSchemeDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:interpretationScheme:info')")
    public Result<InterpretationSchemeDTO> get(@PathVariable("id") Long id){
        InterpretationSchemeDTO data = interpretationSchemeService.get(id);

        return new Result<InterpretationSchemeDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:interpretationScheme:save')")
    public Result<Long> save(@RequestBody InterpretationSchemeDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        interpretationSchemeService.save(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:interpretationScheme:update')")
    public Result<Long> update(@RequestBody InterpretationSchemeDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        interpretationSchemeService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping("batchUpdate")
    @ApiOperation("批量修改")
    public Result batchupdate(@RequestBody InterpretationSchemeDTO[] lists){

        interpretationSchemeService.batchUpdate(lists);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:interpretationScheme:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        interpretationSchemeService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:interpretationScheme:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<InterpretationSchemeDTO> list = interpretationSchemeService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, InterpretationSchemeExcel.class);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取判读方案列表")
    public List<InterpretationSchemeDTO> list() {
        return interpretationSchemeService.list();
    }

    @PostMapping(value = "listByMyInterpretationSchemeId")
    @ApiOperation(value = "获取判读方案列表 依据自定义判读方案id")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "myInterpretationSchemeId", value = "自定义判读方案id", paramType = "query", required = true, dataTypeClass=Long.class)
    })
    public List<InterpretationSchemeDTO> listByMyInterpretationSchemeId(@ApiIgnore @RequestParam Map<String, Object> params) {
        return interpretationSchemeService.listByMyInterpretationSchemeId(params);
    }

}