package com.cnbm.qualityPlanning.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.qualityPlanning.dto.MyInterpretationSchemeDTO;
import com.cnbm.qualityPlanning.excel.MyInterpretationSchemeExcel;
import com.cnbm.qualityPlanning.service.IInterpretationSchemeService;
import com.cnbm.qualityPlanning.service.IMyInterpretationSchemeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 判读方案 表  前端控制器
 *
 * @author why
 * @since  2022-08-15
 */
@RestController
@RequestMapping("/qualityPlanning/myInterpretationScheme")
@Api(tags="自定义判读方案 表")
public class MyInterpretationSchemeController {
    @Autowired
    private IMyInterpretationSchemeService myInterpretationSchemeService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "name", value = "名称", paramType = "query", dataTypeClass = String.class),
        @ApiImplicitParam(name = "code", value = "编码", paramType = "query", dataTypeClass = String.class)
    })
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:myInterpretationScheme:page')")
    public Result<PageData<MyInterpretationSchemeDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<MyInterpretationSchemeDTO> page = myInterpretationSchemeService.page(params);

        return new Result<PageData<MyInterpretationSchemeDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:myInterpretationScheme:info')")
    public Result<MyInterpretationSchemeDTO> get(@PathVariable("id") Long id){
        MyInterpretationSchemeDTO data = myInterpretationSchemeService.get(id);

        return new Result<MyInterpretationSchemeDTO>().ok(data);
    }

    @Autowired
    private IInterpretationSchemeService interpretationSchemeService;

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:myInterpretationScheme:save')")
    public Result<Long> save(@RequestBody MyInterpretationSchemeDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        myInterpretationSchemeService.save(dto);
        //System.out.println(dto.getId());
        //每新增一条自定义判读方案，同时对应新增十条判读方案
        //interpretationSchemeService.batchInsert(dto.getId());

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:myInterpretationScheme:update')")
    public Result<Long> update(@RequestBody MyInterpretationSchemeDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        myInterpretationSchemeService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:myInterpretationScheme:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        myInterpretationSchemeService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('qualityPlanning:myInterpretationScheme:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<MyInterpretationSchemeDTO> list = myInterpretationSchemeService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, MyInterpretationSchemeExcel.class);
    }

}