package com.cnbm.qualityPlanning.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.qualityPlanning.dto.ControlGraphDTO;
import com.cnbm.qualityPlanning.entity.ControlGraph;
import com.cnbm.qualityPlanning.mapper.ControlGraphMapper;
import com.cnbm.qualityPlanning.service.IControlGraphService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 控制图形 表
 *
 * @author why
 * @since 2022-08-15
 */
@Service
public class ControlGraphServiceImpl extends CrudServiceImpl<ControlGraphMapper, ControlGraph, ControlGraphDTO> implements IControlGraphService {

    @Override
    public QueryWrapper<ControlGraph> getWrapper(Map<String, Object> params){

        QueryWrapper<ControlGraph> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotNull(params.get("type")), "type", params.get("type"));

        return wrapper;
    }

    @Override
    public PageData<ControlGraphDTO> page (Map<String, Object> params){
        IPage<ControlGraph> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, ControlGraphDTO.class);
    }

    @Override
    public ControlGraphDTO get(Long id) {
        ControlGraph entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, ControlGraphDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(ControlGraphDTO dto) {
        ControlGraph entity = ConvertUtils.sourceToTarget(dto, ControlGraph.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(ControlGraphDTO dto) {
        ControlGraph entity = ConvertUtils.sourceToTarget(dto, ControlGraph.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }


    @Autowired
    private ControlGraphMapper mapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<ControlGraphDTO> list() {
        List<ControlGraphDTO> list = mapper.list();
        return list;
    }
}