package com.cnbm.qualityPlanning.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.qualityPlanning.dto.InterpretationSchemeDTO;
import com.cnbm.qualityPlanning.entity.InterpretationScheme;
import com.cnbm.qualityPlanning.mapper.InterpretationSchemeMapper;
import com.cnbm.qualityPlanning.service.IInterpretationSchemeService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 判读方案 表
 *
 * @author why
 * @since 2022-08-15
 */
@Service
public class InterpretationSchemeServiceImpl extends CrudServiceImpl<InterpretationSchemeMapper, InterpretationScheme, InterpretationSchemeDTO> implements IInterpretationSchemeService {

    @Autowired
    private InterpretationSchemeMapper mapper;

    @Override
    public QueryWrapper<InterpretationScheme> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");

        QueryWrapper<InterpretationScheme> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);

        return wrapper;
    }

    @Override
    public PageData<InterpretationSchemeDTO> page (Map<String, Object> params){
        IPage<InterpretationScheme> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, InterpretationSchemeDTO.class);
    }

    @Override
    public InterpretationSchemeDTO get(Long id) {
        InterpretationScheme entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, InterpretationSchemeDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(InterpretationSchemeDTO dto) {
        InterpretationScheme entity = ConvertUtils.sourceToTarget(dto, InterpretationScheme.class);
        insert(entity);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsert(Long myInterpretationSchemeId) {
        /*
        List<InterpretationSchemeDTO> lists= new ArrayList<>();
        lists.add(InterpretationSchemeDTO.builder().code(1).ruleNum(1).ruleKey("规则一:有1点落在三倍标准差以外").myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(2).ruleNum(2).ruleKey("规则二:连续x点落在管制中心线同一侧").ruleValue1(6).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(3).ruleNum(3).ruleKey("规则三:连续x点持续上升或下降").ruleValue1(6).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(4).ruleNum(4).ruleKey("规则四:连续x点交替上下跳动").ruleValue1(14).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(5).ruleNum(5).ruleKey("规则五:连续x点中有x点落在中心线同侧两倍标准差以外").ruleValue1(3).ruleValue2(2).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(6).ruleNum(6).ruleKey("规则六:连续x点中有x点落在中心线同侧一倍标准差以外").ruleValue1(5).ruleValue2(4).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(7).ruleNum(7).ruleKey("规则七:连续x点落在中心线两侧的一倍标准差以内").ruleValue1(15).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(8).ruleNum(8).ruleKey("规则八:连续x点落在中心线两侧但未在一倍标准差以内").ruleValue1(8).myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(9).ruleNum(9).ruleKey("规则九:落在规格线以外").myInterpretationSchemeId(myInterpretationSchemeId).build());
        lists.add(InterpretationSchemeDTO.builder().code(10).ruleNum(10).ruleKey("规则十:CPK小于目标值").myInterpretationSchemeId(myInterpretationSchemeId).build());

        for(InterpretationSchemeDTO dto:lists){
            ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);
            save(dto);
        }
         */
        mapper.batchInsert(myInterpretationSchemeId);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(InterpretationSchemeDTO dto) {
        InterpretationScheme entity = ConvertUtils.sourceToTarget(dto, InterpretationScheme.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdate(InterpretationSchemeDTO[] lists) {
        for(InterpretationSchemeDTO dto:lists){
            //效验数据
            ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);
            update(dto);
        }
    }
    
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<InterpretationSchemeDTO> list() {
        List<InterpretationSchemeDTO> list = mapper.list();
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<InterpretationSchemeDTO> listByMyInterpretationSchemeId(Map<String, Object> params) {
        List<InterpretationSchemeDTO> listByMyInterpretationSchemeId = mapper.listByMyInterpretationSchemeId(params);
        return listByMyInterpretationSchemeId;
    }
    
}