package com.cnbm.qualityPlanning.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.qualityPlanning.dto.MyInterpretationSchemeDTO;
import com.cnbm.qualityPlanning.entity.MyInterpretationScheme;
import com.cnbm.qualityPlanning.mapper.MyInterpretationSchemeMapper;
import com.cnbm.qualityPlanning.service.IMyInterpretationSchemeService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Map;

/**
 * 判读方案 表
 *
 * @author why
 * @since 2022-08-15
 */
@Service
public class MyInterpretationSchemeServiceImpl extends CrudServiceImpl<MyInterpretationSchemeMapper, MyInterpretationScheme, MyInterpretationSchemeDTO> implements IMyInterpretationSchemeService {

    @Override
    public QueryWrapper<MyInterpretationScheme> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");
        String code = (String)params.get("code");

        QueryWrapper<MyInterpretationScheme> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), "name", name);
        wrapper.like(StringUtils.isNotBlank(code), "code", code);

        return wrapper;
    }

    @Override
    public PageData<MyInterpretationSchemeDTO> page (Map<String, Object> params){
        IPage<MyInterpretationScheme> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        return getPageData(page, MyInterpretationSchemeDTO.class);
    }

    @Override
    public MyInterpretationSchemeDTO get(Long id) {
        MyInterpretationScheme entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, MyInterpretationSchemeDTO.class);
    }

    @Autowired
    private InterpretationSchemeServiceImpl interpretationSchemeService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(MyInterpretationSchemeDTO dto) {
        MyInterpretationScheme entity = ConvertUtils.sourceToTarget(dto, MyInterpretationScheme.class);
        insert(entity);
        //每新增一条自定义判读方案，同时对应新增十条判读方案
        interpretationSchemeService.batchInsert(entity.getId());
        //System.out.println(dto);
        //System.out.println(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(MyInterpretationSchemeDTO dto) {
        MyInterpretationScheme entity = ConvertUtils.sourceToTarget(dto, MyInterpretationScheme.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

}